// ==========================================================
//  AUTHOR:  R.P. Wiegand        CREATED: 14-Mar-06
// ----------------------------------------------------------
//  This module provides the class definition for the class
// that implements a basic binary tree node.  
// ==========================================================


public class BNode {
  // Technically these should be protected ... but to
  // keep things simple in the insert() and search() methods pf
  // BTree.java, I make them public.  In general, external
  // access should use the left() and right() methods, which
  // return safe copies of the node.
  public BNode pLeft;
  public BNode pRight;
  protected BNode pParent;
  protected int key;

  // Consutrctors
  public BNode() {
    defaultNode();
  }

  public BNode(int key) {
    defaultNode();
    this.key = key;
  }

  public BNode(BNode pLeft, BNode pRight, BNode pParent, int key) {
    this.pLeft = pLeft;
    this.pRight = pRight;
    this.pParent = pParent;
    this.key = key;
  }


  // Accessor & general helper methods
  public BNode clone() {
    BNode bn = new BNode(this.pLeft,this.pRight,this.pParent,this.key);
    return bn;
  }

  protected void defaultNode() {
    pLeft = null;
    pRight = null;
    pParent = null;
    key = 0;
  }

  public int getKey() {return(key);}
  public BNode left() {return pLeft.clone();}
  public BNode right() {return pRight.clone();}
  public BNode parent() {return pParent.clone();}


//------------------------------------------------
//   Convenience methods for linking child and
// parent nodes.
//------------------------------------------------
 
//   The linkLeft and linkRight methods each
// attach the x ptr to the appropriate pLeft
// or pRight link, and MAY link x's parent
// ptr back to the current node.  This upLink
// happens only if the parent exist and the
// calling routine specifies upLink = 1 (which
// is the default). 

  public void linkLeft  (BNode x, boolean upLink) {
    pLeft = x;   
    if ( (x != null) && (upLink) ) x.pParent = this;
  }

  public void linkRight  (BNode x, boolean upLink) {
    pRight = x;   
    if ( (x != null) && (upLink) ) x.pParent = this;
  }

  public void linkParent(BNode x) {
    pParent = x;
  }

  public void linkFull  (BNode l, BNode r, BNode p) {
    linkLeft(l,false);
    linkRight(r,false);
    linkParent(p);
  }



  // Methods & member variables for dealing the the compare count global
  protected static int compareCount = 0;

  public static void resetCompareCount() {
    compareCount = 0;
  }

  public static int getCompareCount() {
    return (compareCount);
  }


//------------------------------------------------
//   Methods which have to do with comparing   
// keys in the node.  Use these methods to do any
// key comparisons, and the mCount variable will
// be appropriately updated.
//
//   They each return a -1, 0, or 1:
// -1   Current node's key is LESS THAN inputted key
//  0   Current node's key is EQUAL TO inputted key
// +1   Current node's key is GREATER THAN inputted key
//------------------------------------------------
  public int compareKeys(BNode x) {
    return compareKeys(x.getKey());
  }

  public int compareKeys(int key) {
    int compVal = 0;
  
    if (this.key < key) compVal = -1;
    else if (this.key > key) compVal = 1;

    compareCount++;
  
    return(compVal);
  }

}


