// ==========================================================
//  AUTHOR:  R.P. Wiegand        CREATED: 14-Mar-06
// ----------------------------------------------------------
//  This module provides the class definition for the class
// that implements a basic binary tree.  
// ==========================================================

public class BTree {
  protected BNode pRoot;
  protected int nodeCount;

  // Constructors
  public BTree() {
    pRoot      = null;
    nodeCount = 0;
  }

  // Accessor & Helper methods
  public int  getNodeCount() {return(nodeCount);}

  //------------------------------------------------
  //   Print routines for rendering the tree.  
  // There is a wrapper printTree() procedure that
  // relies on the recursive procedure truePrintTree()
  // as well as the helper routine printNode().
  //
  //   The routines render the tree side-ways...so
  // that right is on top and left is on bottom.
  // The tree is oriented with the root on the left.
  //------------------------------------------------
  public void printTree() {
    if (pRoot == null) System.out.println("Tree empty!");
    else printSubtree(pRoot,0);
  }

  protected void printSubtree(BNode curr, int level) {
    if (curr != null) {
      printSubtree(curr.pRight,level+1);
      printNode(curr,level);
      printSubtree(curr.pLeft,level+1);
    }
  }

  protected void printNode(BNode x, int level) {
    for (int i=0; i<level; i++)
      System.out.print("\t");
    System.out.println(x.getKey());
  }

  
  // Basic Tree Operations
  public void insert(int key) {
    BNode x = new BNode(key);
    insert(x);
  }

  public void insert(BNode x) {
    BNode pLast = null;
    BNode pCurr = pRoot;
    int comparison = 0;

    while (pCurr != null) {
      pLast = pCurr;
      comparison = x.compareKeys(pCurr);
      if (comparison > 0) pCurr = pCurr.pRight;
      else                pCurr = pCurr.pLeft;
    }

    // rely on comparison having already been set

    nodeCount++;

    if (pLast == null) pRoot = x;

    else {
      x.linkParent(pLast);
      if (comparison > 0) pLast.linkRight(x,true);
      else                pLast.linkLeft(x,true);
    }
  }

  public BNode search(int key) {
    BNode pCurr = pRoot;
    boolean keyFound = false;

    while ( (pCurr != null) && (!keyFound) ) {
      int comparison = pCurr.compareKeys(key);

      if (comparison == 0) keyFound = true;
      else if (comparison == -1) pCurr = pCurr.pRight;
      else if (comparison == 1) pCurr = pCurr.pLeft;
      else System.err.println("ERROR:  compareKeys() returned something other than -1, 0 , 1");
    }

    if (!keyFound) return null;

    return pCurr.clone();
  }//search


  // The main of the class is used to unit test the tree...
  public static void main(String args[]) {
    BTree bt = new BTree();
 
    System.out.println("TESTING INSERT:");
    System.out.println("  Inserting:  5, 2, 6, 1, 4");
  
    bt.insert(5);
    bt.insert(2);
    bt.insert(6);
    bt.insert(1);
    bt.insert(4);

    System.out.println("");  
    System.out.println("  Your tree should look like this:");
    System.out.println("\t6"); 
    System.out.println("5"); 
    System.out.println("\t\t4"); 
    System.out.println("\t2"); 
    System.out.println("\t\t1"); 
    System.out.println();
    System.out.println("  Your tree:");
    bt.printTree();
    System.out.println();
    System.out.println("  Should have needed 6 key comparisons.");  
    System.out.println("  Your tree needed " + BNode.getCompareCount() + " comparisons.");
    System.out.println("  Your tree believes it has " + bt.getNodeCount() + " nodes in it.");
    // Declare some temp vars we will ndeed
    BNode x;
    BNode y;

    System.out.println("");
    System.out.println("TESTING SEARCH: ");  
    BNode.resetCompareCount();  
    x = bt.search(6);
    System.out.println("  Searching for 6: " + (x!=null?"[found]":"[not found]") + "  (should be found)");
    x = bt.search(8);
    System.out.println("  Searching for 8: " + (x!=null?"[found]":"[not found]") + " (shouldn't be found)");
    System.out.println("  Should have needed 4 key comparisons.");  
    System.out.println("  Your tree needed " + BNode.getCompareCount() + " comparisons.");
    System.out.println();

    BNode.resetCompareCount();  
    x = bt.search(1);
    System.out.println("  Searching for 1: " + (x!=null?"[found]":"[not found]") + "  (should be found)");
    x = bt.search(3);
    System.out.println("  Searching for 3: " + (x!=null?"[found]":"[not found]") + "  (shouldn't be found)");
    System.out.println("  Should have needed 6 key comparisons.");  
    System.out.println("  Your tree needed " + BNode.getCompareCount() + " comparisons.");
    System.out.println();  
  }

}
