// ==========================================================
//  AUTHOR:  R.P. Wiegand        CREATED: 14-Mar-06
// ----------------------------------------------------------
//   This is the main program for the project.  It accepts
// three command line parameters:  
//   - file name for the insert keys
//   - file name for the search keys
//
//  The code here is simple but catches several major, obvious
// errors (no file names specified, file cannot open, etc.).
// ==========================================================
import java.util.Vector;
import java.io.StreamTokenizer;
import java.io.FileReader;

public class Proj2 {

  static boolean insertTest(int data[], BTree bt) {
    System.out.println("Inserting nodes...");

    BNode.resetCompareCount();
    for (int i=0; i<data.length; i++) {
      System.out.println("Inserting " + data[i]);
      bt.insert(data[i]);
    }
    int count = BNode.getCompareCount();

    System.out.println("INSERT: " + count + " " +
		       bt.getNodeCount());

    return (bt.getNodeCount() == data.length);
  }


  static boolean searchTest(int data[], BTree bt) {
    System.out.println("Searching for nodes...");

    BNode.resetCompareCount();
    for (int i=0; i<data.length; i++) {
      System.out.print("Searching for " + data[i]);
      BNode bn = bt.search(data[i]);
      System.out.println(bn!=null?"[found]":"[not found]");
    }
    int count = BNode.getCompareCount();

    System.out.println("SEARCH: " + count + " " +
		       bt.getNodeCount());

    return (bt.getNodeCount() == data.length);
  }


  public static int[] readDataFile(String fileName) {
    System.out.println("Reading from file " + fileName + " ...");

    // Read file into a vector
    Vector v = new Vector();
    try {
      FileReader fr = new FileReader(fileName);
      StreamTokenizer tokenizer = new StreamTokenizer(fr);
      int i = 0;
      boolean fileRead = false;
      while (!fileRead) {
	int token = tokenizer.nextToken();
	if (token == StreamTokenizer.TT_NUMBER) {
	  Double x = new Double(tokenizer.nval);
	  if (!v.add(x)) {
	    System.err.println("Stream tokenizer could not convert item from file " + fileName);
	    System.exit(1);
	  }
	}
	else if (token == StreamTokenizer.TT_EOF) 
	  fileRead = true;
      }
      fr.close();
    } catch (Exception e) {
      System.err.println("Error reading file " + fileName);
      System.err.println(e);
      System.exit(1);
    }

    // Pull it out into an array for simplicity and speed
    int data[] = new int[v.size()];
    for (int i=0; i<v.size(); i++) {
      Double x = (Double)v.get(i);
      data[i] = (int)x.doubleValue();
    }

    return data;
  }


  public static void main(String args[]) {
    if (args.length < 2) {
      System.err.println("Incorrect usage.  You must specify two command line parameters:");
      System.err.println("  - The file name for the insert data set");
      System.err.println("  - The file name for the search data set"); 
      System.err.println("  java Proj2 <insertfile> <searchfile>");
      System.err.println();
      System.exit(1);
    }

    String insertFileName = args[0];
    String searchFileName = args[1];

    System.out.println("Reading inserts from " + insertFileName);
    System.out.println("Reading searches from " + searchFileName);

    int insertData[] = readDataFile(insertFileName);
    int searchData[] = readDataFile(searchFileName);

    // Main code here
    BTree bt = new BTree();
    if (!insertTest(insertData,bt)) return;
    bt.printTree();
    if (!searchTest(searchData,bt)) return;

    System.out.println("Inserts & Searches completed.");
  }
}
