#!/bin/sh

# This script will take 3 parameters:  max data size, number of data elements,
# and the number of trials to run.  It will call runExp.sc for the N specified
# trials, then collate the data with genData.sc.  The results should all be
# in the directory specified by DATA_DIR (below).  The directory will be
# created if it does not already exist.

#
# Populate ENV vars
#   N         - Number of trials to perform
#   DATA_DIR  - location of data input files
#
N=$3
DATA_DIR=./data


#
# Generate an error of there are fewer than 3 command line arguments
#
if [ $# -lt 3 ] ; then
  echo "$0 takes 3 arguments:  max data size, number of data elements, and number of trials!"
  echo " "
  exit
fi


#
# Create the data directory, if it does not already exist
#
if [ ! -d $DATA_DIR ] ; then
  echo "Making $DATA_DIR..."
  mkdir $DATA_DIR
fi


echo "Running $N trials for max size $1, data range $2."
echo "Data placed in $DATA_DIR"
echo " "


#
# Run the experiment for the specified number of trials
#
TRIAL=0
while test $TRIAL -lt $N
do
  echo "  Running $TRIAL trial"
  runExp.sc $1 $2 $TRIAL $DATA_DIR
  TRIAL=`expr $TRIAL + 1`
done

#
# Collate the data into .final files
#
genData.sc $1 $2 $DATA_DIR


